package com.fmi.bytecode.annotations.tool.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.fmi.bytecode.annotations.element.AnnotationRecord;
import com.fmi.bytecode.annotations.element.ClassInfo;
import com.fmi.bytecode.annotations.element.impl.ClassInfoImpl;
import com.fmi.bytecode.annotations.file.FileInfo;
import com.fmi.bytecode.annotations.tool.ReadResult;
import com.fmi.bytecode.annotations.tool.ReadingException;

/**
 * @author Krasimir Topchiyski
 *
 */
public class ReadResultImpl implements ReadResult {

	private List<FileInfo> files = new ArrayList<FileInfo>();
	private Map<String, List<AnnotationRecord>> classLevelAnnotations = new HashMap<String, List<AnnotationRecord>>();	
	private Map<String, ClassInfo> classes = new HashMap<String, ClassInfo>();
	private List<String> problems = new ArrayList<String>();
	
	public ReadResultImpl() {
	}
	
	public FileInfo[] getProcessedFiles() {
		return files.toArray(new FileInfo[files.size()]);
	}
    
    public void addProcessedFile(FileInfo fi) {    	
    	files.add(fi);
    }

	public void setProcessedFiles(FileInfo[] files) {
        this.files.clear();
        if(files!=null){            
            for(FileInfo fi: files){
                addProcessedFile(fi);
            }
        }
	}

    public Map<String, List<AnnotationRecord>> getClassLevelAnnotations() {
    	return Collections.unmodifiableMap(classLevelAnnotations);
    }
    
    public void addClassLevelAnnotation(AnnotationRecord a) {
    	List<AnnotationRecord> list = classLevelAnnotations.get(a.getTypeName());
    	if(list == null){
    		list = new ArrayList<AnnotationRecord>();
    		classLevelAnnotations.put(a.getTypeName(),list);
    	}
    	list.add(a);
    }

    public void addClassLevelAnnotations(AnnotationRecord[] anns) {
    	for(AnnotationRecord a: anns){
    		addClassLevelAnnotation(a);
    	}
    }

    public void setClassLevelAnnotations(AnnotationRecord[] anns) {
        classLevelAnnotations.clear();
        if(anns!=null){            
            for(AnnotationRecord a: anns){
                addClassLevelAnnotation(a);
            }
        }    	
    }

    public Map<String, ClassInfo> getClasses() {
    	return Collections.unmodifiableMap(classes);
    }

    public ClassInfo getClass(String className) {
    	ClassInfo ci = classes.get(className);
    	return ci;
    }
    
    public void addClass(ClassInfo ci) throws ReadingException {
    	ClassInfoImpl old = (ClassInfoImpl)classes.get(ci.getName());
    	if(old!=null){
			FileInfo[] fi = old.getContainingFiles();
			String files = "";
			if(fi!=null){
				for(FileInfo f: fi){
					files+=(f.getName()+" , ");
				}
			}
			String newFile = "class stream";
    		FileInfo[] cfs = ci.getContainingFiles();
        	if((cfs!=null) && (cfs.length>0)){
        		old.addContainingFile(cfs[0]);
        		newFile = cfs[0].getName();
        	}
    		
        	if(!old.equals(ci)){
        		problems.add( "There are different versions of the class ["+ci.getName()+"]! "+
    					" One version can be found in: "+files+
    					" And the other in: "+newFile);
        	}
    		
    	} else {
    		classes.put(ci.getName(), ci);
    	}
    }
    
    public void addClasses(ClassInfo[] cs)  throws ReadingException {
    	for(ClassInfo ci: cs){
    		addClass(ci);
    	}
    }
    
    public void setClasses(ClassInfo[] cis)  throws ReadingException {
        classes.clear();
        if(cis!=null){            
            for(ClassInfo ci: cis){
                addClass(ci);
            }
        }    	
    }

	public String[] getProcessingProblems() {		
		return problems.toArray(new String[problems.size()]);
	}

}
