import java.io.File;
import java.util.Arrays;
import java.util.Map;

import javax.ejb.EJB;
import javax.ejb.EJBs;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;

import com.fmi.bytecode.annotations.element.AnnotationRecord;
import com.fmi.bytecode.annotations.element.ClassInfo;
import com.fmi.bytecode.annotations.element.ConstructorInfo;
import com.fmi.bytecode.annotations.element.ElementInfo;
import com.fmi.bytecode.annotations.element.FieldInfo;
import com.fmi.bytecode.annotations.element.NamedMember;
import com.fmi.bytecode.annotations.element.MethodInfo;
import com.fmi.bytecode.annotations.input.ByteCodeReader;
import com.fmi.bytecode.annotations.input.ByteCodeReaderFactory;
import com.fmi.bytecode.annotations.tests.jeeapplication.AnnotationToolTestBeanClass;
import com.fmi.bytecode.annotations.tests.jeeapplication.AnnotationToolTestBeanSuperClass;
import com.fmi.bytecode.annotations.tests.jeeapplication.AnnotationToolTestLocalInterface1;
import com.fmi.bytecode.annotations.tests.jeeapplication.AnnotationToolTestLocalInterface2;
import com.fmi.bytecode.annotations.tests.jeeapplication.AnnotationToolTestRemoteInterface1;
import com.fmi.bytecode.annotations.tests.jeeapplication.AnnotationToolTestRemoteInterface2;
import com.fmi.bytecode.annotations.tests.jeeapplication.Interface1;
import com.fmi.bytecode.annotations.tests.jeeapplication.Interface2;
import com.fmi.bytecode.annotations.tests.jeeapplication.ParameterAnnotation;
import com.fmi.bytecode.annotations.tool.ClassInfoReader;
import com.fmi.bytecode.annotations.tool.ReadResult;
import com.fmi.bytecode.annotations.tool.AnnotationsReaderFactory;


public class TestsRunner {
	
	private static String INPUT_ROOT_DIRECTORY_NAME = ".";
	private static File INPUT_ROOT_DIRECTORY = new File(INPUT_ROOT_DIRECTORY_NAME);
	
	public static void main(String args[]) {
		try {
			ByteCodeReader bcReader = ByteCodeReaderFactory.createByteCodeReader();
			//ByteCodeReader bcReader = ByteCodeReaderFactory.createByteCodeReader(
					//"com.fmi.bytecode.annotations.input.adapters.asm30.ByteCodeReaderImpl");
			
			ClassInfoReader cif = AnnotationsReaderFactory.getReader(bcReader);
			ReadResult readResult = cif.read(new File[]{INPUT_ROOT_DIRECTORY});
			runTests(readResult);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	private static void runTests(ReadResult readResult) throws TestException {
		testBeanClassAnnotations(readResult);
	}

	private static void testBeanClassAnnotations(ReadResult readResult) throws TestException {
		
		log("Test: testBeanClassAnnotations() ...");
		
		ClassInfo beanClassInfo= readResult.getClass(AnnotationToolTestBeanClass.class.getName());
		System.out.println(beanClassInfo);
		//Base class checks
		checkClassPackage(AnnotationToolTestBeanClass.class.getPackage().getName(), beanClassInfo);
		checkElementName(AnnotationToolTestBeanClass.class.getName(), beanClassInfo);
		checkInterfaces(new String[]{}, beanClassInfo);
		
		//Class level annotations checks
		AnnotationRecord statelessAnnotation = checkAnnotationExistence(Stateless.class.getName(), beanClassInfo);
		NamedMember beanName = checkAnnotationMemberExistence("name", statelessAnnotation);
		checkAnnotationStringMemberValue("AnnotationToolTestBean", beanName);
		
		AnnotationRecord localAnnotation = checkAnnotationExistence(Local.class.getName(), beanClassInfo);
		NamedMember localInterfaces = checkAnnotationMemberExistence("value", localAnnotation);
		checkAnnotationStringArrayMemberValue(
				new String[] {AnnotationToolTestLocalInterface1.class.getName(), AnnotationToolTestLocalInterface2.class.getName()}, localInterfaces);

		AnnotationRecord remoteAnnotation = checkAnnotationExistence(Remote.class.getName(), beanClassInfo);
		NamedMember remoteInterfaces = checkAnnotationMemberExistence("value", remoteAnnotation);
		checkAnnotationStringArrayMemberValue(
				new String[] {AnnotationToolTestRemoteInterface1.class.getName(), AnnotationToolTestRemoteInterface2.class.getName()}, remoteInterfaces);
	
		AnnotationRecord ejbsAnnotation = checkAnnotationExistence(EJBs.class.getName(), beanClassInfo);
		NamedMember ejbsArray = checkAnnotationMemberExistence("value", ejbsAnnotation);
		checkAnnotationArrayLength(2, ejbsArray);
		NamedMember[] ejbs  = ejbsArray.getMemberArrayValue();
		
		AnnotationRecord ejb1 = ejbs[0].getAnnotationValue();
		NamedMember beanNameMember1 = checkAnnotationMemberExistence("beanName", ejb1);
		checkAnnotationStringMemberValue("beanName1", beanNameMember1);
		NamedMember beanInterfaceMember1 = checkAnnotationMemberExistence("beanInterface", ejb1);
		checkAnnotationStringMemberValue(AnnotationToolTestLocalInterface1.class.getName(), beanInterfaceMember1);
		
		AnnotationRecord ejb2 = ejbs[1].getAnnotationValue();
		NamedMember beanNameMember2 = checkAnnotationMemberExistence("beanName", ejb2);
		checkAnnotationStringMemberValue("beanName2", beanNameMember2);
		NamedMember beanInterfaceMember2 = checkAnnotationMemberExistence("beanInterface", ejb2);
		checkAnnotationStringMemberValue(AnnotationToolTestLocalInterface2.class.getName(), beanInterfaceMember2);

		//Fields checks
		FieldInfo ejbRefFied = verifyFieldExistence("referenceToLocalBusinessInterface", beanClassInfo);
		verifyFieldType(AnnotationToolTestLocalInterface1.class.getName(), ejbRefFied);
		AnnotationRecord ejbRefAnnotation = checkAnnotationExistence(EJB.class.getName(), ejbRefFied);
		NamedMember beanNameMember3 = checkAnnotationMemberExistence("beanName", ejbRefAnnotation);
		checkAnnotationStringMemberValue("beanName2", beanNameMember3);
		
		//Methods checks
		MethodInfo businessMethod1 = verifyNoArgsMethodReturnType("void", "businessMethod1", beanClassInfo);
		AnnotationRecord transactionAttribute1 = checkAnnotationExistence(TransactionAttribute.class.getName(), businessMethod1);
		NamedMember transAttrMember1 = checkAnnotationMemberExistence("value", transactionAttribute1);
		checkAnnotationEnumMemberValue(TransactionAttributeType.REQUIRED.name(), transAttrMember1);

		verifyArgsMethodExistence("businessMethod2",
				new String[] {"boolean", "char", "byte", "int", "long", "float", "double",
					Object.class.getName(), AnnotationToolTestLocalInterface1.class.getName() },
				beanClassInfo);
		
		verifyNoArgsMethodReturnType("int", "businessMethod3", beanClassInfo);
		verifyNoArgsMethodReturnType(AnnotationToolTestLocalInterface2.class.getName(), "businessMethod4", beanClassInfo);

		MethodInfo businessMethod5 = verifyArgsMethodExistence("businessMethod5",
				new String[] {"java.lang.Object"},
				beanClassInfo);
		AnnotationRecord parameterAnnotation = verifyParameterAnnotationExsistence(
				ParameterAnnotation.class.getName(), 0, businessMethod5);
		NamedMember value1 = checkAnnotationMemberExistence("value1", parameterAnnotation);
		checkAnnotationStringMemberValue("actualValue1", value1);
		NamedMember value2 = checkAnnotationMemberExistence("value2", parameterAnnotation);
		checkAnnotationStringMemberValue("actualValue2", value2);
		
		//Constructors checks
		verifyNoArgsConstructorExistence(beanClassInfo);
		
		//Super class name check
		checkStringValue(AnnotationToolTestBeanSuperClass.class.getName(), beanClassInfo.getSuperclassName());
		
		
		//Bean super class checks
		ClassInfo beanSuperClassInfo = beanClassInfo.getSuperclass();
		
		//Base class checks
		checkClassPackage(AnnotationToolTestBeanSuperClass.class.getPackage().getName(), beanSuperClassInfo);
		checkElementName(AnnotationToolTestBeanSuperClass.class.getName(), beanSuperClassInfo);
		checkInterfaces(new String[]{Interface1.class.getName(), Interface2.class.getName()}, beanSuperClassInfo);
		
		//Methods checks
		MethodInfo businessMethod0 = verifyNoArgsMethodReturnType("void", "businessMethod0", beanSuperClassInfo);
		AnnotationRecord transactionAttribute0 = checkAnnotationExistence(TransactionAttribute.class.getName(), businessMethod0);
		NamedMember transAttrMember0 = checkAnnotationMemberExistence("value", transactionAttribute0);
		checkAnnotationEnumMemberValue(TransactionAttributeType.REQUIRES_NEW.name(), transAttrMember0);

		log("Test: testBeanClassAnnotations() finished successfully.");
	}

	private static AnnotationRecord verifyParameterAnnotationExsistence(String annotationName,
			int parameterIndex, MethodInfo method) throws TestException {
		Map<String,AnnotationRecord>[] parameterAnnotations = method.getParameterAnnotations();
		if (parameterAnnotations == null || parameterAnnotations.length <= parameterIndex) {
			throw new TestException("Method " + method.getName() +
					" has " + parameterAnnotations.length + " parameter annotations. Checked annotation index is " +
					parameterIndex + ". There is no expected annotation there " + annotationName);
		}
		
		AnnotationRecord paramAnn = parameterAnnotations[parameterIndex].get(annotationName);
		if (paramAnn == null) {
			throw new TestException("Method " + method.getName() +
					" has no parameter annotation " + annotationName +
					" at parameter index " + parameterIndex + ".");
		}

		log("Test: Parameter annotation '" + annotationName + "' at parameter index " +
				parameterIndex + " of method " + method.getName() + " -> OK");
		return paramAnn;
	}

	private static void checkInterfaces(String[] values, ClassInfo classInfo) throws TestException {
		String[] interfaceNames = classInfo.getInterfacesNames();
		if (interfaceNames.length != values.length) {
			throw new TestException("Class " + classInfo.getName() + " has incorrect interfaces length " +
					interfaceNames.length + ". Expected value is " + values.length);
		}
		for (int i=0; i<interfaceNames.length; i++) {
			if (!values[i].equals(interfaceNames[i])) {
				throw new TestException("Class " + classInfo.getName() + " has wrong interfaces definition at index " + i +
						". Founded is " + interfaceNames[i] + " but expected is " + values[i]);
			}
		}
		log("Test: Class '" + classInfo.getName() + "' interfaces " + Arrays.toString(values) + " -> OK");
	}

	private static void verifyFieldType(String type, FieldInfo ejbRefFied) throws TestException {
		if (!ejbRefFied.getType().equals(type)) {
			throw new TestException("Field " + ejbRefFied.getName() +
					" has wrong type " + ejbRefFied.getType() + ". Expected type is " + type);
		}
		log("Test: Field type of '" + ejbRefFied.getName() + "' field is '" + type + "' -> OK");
	}

	private static FieldInfo verifyFieldExistence(String fieldName, ClassInfo classInfo) throws TestException {
		FieldInfo field = classInfo.getField(fieldName);
		if (field == null) {
			throw new TestException("Field " + fieldName +
					" does not exists in class " + classInfo.getName());
		}
		log("Test: Field '" + fieldName + "' exists in class '" + classInfo.getName() + "' -> OK");
		return field;
	}

	private static void checkAnnotationEnumMemberValue(String value, NamedMember member) throws TestException {
		if (!member.getEnumConstantValue().getEnumerationLiteral().equals(value)) 
			throw new TestException("Annotation member " + member +
					" has wrong value: " + member.getEnumConstantValue().getEnumerationLiteral() +
					". Expected value is " + value);
		log("Test: Annotation member '" + member + "' value '" + value + "' -> OK");
	}

	private static MethodInfo verifyNoArgsMethodExistence(String methodName, ClassInfo classInfo) throws TestException {
		MethodInfo method = classInfo.getMethod(methodName, new String[]{});
		if (method == null) {
			throw new TestException("Method '" + methodName +
					"' without args does not exists in class " + classInfo.getName());
		}
		log("Test: Method '" + methodName + "' exists in class '" + classInfo.getName() + "' -> OK");
		return method;
	}
	
	private static MethodInfo verifyNoArgsMethodReturnType(String returnType, String methodName, ClassInfo classInfo) throws TestException {
		MethodInfo method = verifyNoArgsMethodExistence(methodName, classInfo);
		if (!method.getReturnType().equals(returnType)) {
			throw new TestException("Method '" + methodName +
					"' has wrong return type. Founded type is " + method.getReturnType() + " but expected is " + returnType);
		}
		log("Test: Method '" + methodName + "' has correct return type '" + returnType + "' -> OK");
		return method;
	}
	
	private static MethodInfo verifyArgsMethodExistence(String methodName, String[] methodArgs, ClassInfo classInfo) throws TestException {
		MethodInfo method = classInfo.getMethod(methodName, methodArgs);
		if (method == null) {
			throw new TestException("Method " + methodName +
					" does not exists in class " + classInfo.getName());
		}
		String[] params = method.getParameters();
		if (params.length != methodArgs.length) {
			throw new TestException("Method " + methodName +
					" has wrong size of paranmeters " + params.length + ". Expected length is " + methodArgs.length);
		}
		
		for (int i=0; i<params.length; i++) {
			if (!methodArgs[i].equals(params[i])) {
				throw new TestException("Method " + methodName +
						" has wrong param definition at index " + i +
						". Founded is " + params[i] + " but expected is " + methodArgs[i]);
			}
		}
		log("Test: Method '" + methodName + "' with args " + Arrays.toString(params) + " exists in class '" + classInfo.getName() + "' -> OK");
		return method;
	}

	private static ConstructorInfo verifyNoArgsConstructorExistence(ClassInfo classInfo) throws TestException {
		ConstructorInfo constructor = classInfo.getConstructor(new String[]{});
		if (constructor == null) {
			throw new TestException("Constructor without args does not exists in class " + classInfo.getName());
		}
		return constructor;
	}

	private static void checkStringValue(String value1, String value2) throws TestException {
		if (!value1.equals(value2))
			throw new TestException("Wrong string value. Expected value is " + value1 +
					" but actual " + value2);
		log("Test: String value equality " + value1 + " -> OK");
	}

	private static void checkClassPackage(String name, ClassInfo classInfo) throws TestException {
		if (classInfo.getPackage() == null || !classInfo.getPackage().equals(name))
			throw new TestException("Package name " +
					"is wrong. Expected name is " + name + " but actual " + classInfo.getPackage());
		log("Test: Package name of class " + classInfo.getName() + " -> OK");
	}

	private static void checkElementName(String name, ElementInfo elementInfo) throws TestException {
		if (elementInfo.getName() == null || !elementInfo.getName().equals(name))
			throw new TestException("Bytecode element " +
					"has wrong name. Expected name is " + name + " but actual " + elementInfo.getName());
		log("Test: Element name of element " + elementInfo.getName() + " -> OK");
	}

	private static AnnotationRecord checkAnnotationExistence(String annotationName, ElementInfo element)
		throws TestException {
		AnnotationRecord annotation = element.getAnnotation(annotationName);
		if (annotation == null)
			throw new TestException("The " + annotationName +
				" annotation of class " + element.getName() +
				" does not exists in the output result.");
		log("Test: Annotation '" + annotationName + "' existence in element '" + element.getName() + "' -> OK");
		return annotation;
	}
	
	private static NamedMember checkAnnotationMemberExistence(String memberName, AnnotationRecord annotation)
		throws TestException {
		NamedMember member = annotation.getMember(memberName);
		if (member == null) 
			throw new TestException("The " + memberName +
				" attribute of " + annotation.getTypeName() +
				" annotation of bytecode element " + annotation.getOwner().getName() +
				" does not exists in the output result.");
		log("Test: Annotation member '" + memberName + "' existence in annotation '" + annotation.getTypeName() + "' -> OK");
		return member;
	}
	
	private static void checkAnnotationStringMemberValue(String value, NamedMember member)
		throws TestException {
		if (!member.getMemberValue().equals(value)) 
			throw new TestException("Annotation member " + member +
					" has wrong value: " + member.getMemberValue() + ". Expected value is " + value);
		log("Test: Annotation member '" + member + "' value '" + value + "' -> OK");
	}
	
	private static void checkAnnotationStringArrayMemberValue(String[] value, NamedMember member)
		throws TestException {
		checkAnnotationArrayLength(value.length, member);
		NamedMember[] members = member.getMemberArrayValue();
		for (int i=0; i<value.length; i++) {
			String curValue = value[i];
			NamedMember curMember = members[i];
			checkAnnotationStringMemberValue(curValue, curMember);
		}
		log("Test: Annotation string array member '" + Arrays.toString(value) + "' value -> OK");
	}

	private static void checkAnnotationArrayLength(int expectedLength, NamedMember member) throws TestException {
		int length = member.getMemberArrayLength();
		if (length != expectedLength) {
			throw new TestException("Annotation array member with wrong length. Expected length is " + expectedLength + " but actual " + length);
		}
		log("Test: Annotation array member '" + member.getName() + "' length " + expectedLength + "' -> OK");
	}
	
	private static final void log(String message) {
		System.out.println(message);
	}
}
